@@ -1,4 +1,4 @@
-# $Id: Gtk.pm 2447 2009-02-17 05:04:43Z rcaputo $
+# $Id: Gtk.pm 2219 2007-08-19 00:54:55Z rcaputo $
 
 # Gtk-Perl event loop bridge for POE::Kernel.
 
@@ -11,7 +11,7 @@ use strict;
 use POE::Loop::PerlSignals;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2447 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2219 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 =for poe_tests
 
@@ -93,7 +93,7 @@ sub loop_attach_uidestroy {
         $self->_dispatch_event
           ( $self, $self,
             EN_SIGNAL, ET_SIGNAL, [ 'UIDESTROY' ],
-            __FILE__, __LINE__, undef, time(), -__LINE__
+            __FILE__, __LINE__, time(), -__LINE__
           );
       }
       return 0;
@@ -328,7 +328,7 @@ __END__
 
 =head1 NAME
 
-POE::Loop::Gtk - a bridge that allows POE to be driven by Gtk
+POE::Loop::Gtk - a bridge that supports Gtk's event loop from POE
 
 =head1 SYNOPSIS
 
@@ -336,13 +336,13 @@ See L<POE::Loop>.
 
 =head1 DESCRIPTION
 
-POE::Loop::Gtk implements the interface documented in L<POE::Loop>.
-Therefore it has no documentation of its own.  Please see L<POE::Loop>
-for more details.
+This class is an implementation of the abstract POE::Loop interface.
+It follows POE::Loop's public interface exactly.  Therefore, please
+see L<POE::Loop> for its documentation.
 
 =head1 SEE ALSO
 
-L<POE>, L<POE::Loop>, L<Gtk>, L<POE::Loop::PerlSignals>
+L<POE>, L<POE::Loop>, L<Gtk>
 
 =head1 AUTHORS & LICENSING
 
@@ -352,4 +352,4 @@ and POE's licensing.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
-# TODO - Edit.
+# TODO - Redocument.
