@@ -1,3 +1,5 @@
+# $Id: IO_Poll.pm 2222 2007-08-19 05:02:19Z rcaputo $
+
 # IO::Poll event loop bridge for POE::Kernel.  The theory is that this
 # will be faster for large scale applications.  This file is
 # contributed by Matt Sergeant (baud).
@@ -6,7 +8,7 @@
 package POE::Loop::IO_Poll;
 
 use vars qw($VERSION);
-$VERSION = '1.294_512'; # NOTE - Should be #.### (three decimal places)
+$VERSION = do {my($r)=(q$Revision: 2222 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 # Include common signal handling.
 use POE::Loop::PerlSignals;
@@ -88,23 +90,18 @@ sub loop_attach_uidestroy {
 }
 
 #------------------------------------------------------------------------------
-# Maintain time watchers.  For this loop, we simply save the next
-# event time in a scalar.  loop_do_timeslice() will use the saved
-# value.  A "paused" time watcher is just a timeout for some future
-# time.
-
-my $_next_event_time = time();
+# Maintain time watchers.
 
 sub loop_resume_time_watcher {
-  $_next_event_time = $_[1];
+  # does nothing
 }
 
 sub loop_reset_time_watcher {
-  $_next_event_time = $_[1];
+  # does nothing
 }
 
 sub loop_pause_time_watcher {
-  $_next_event_time = time() + 3600;
+  # does nothing
 }
 
 # A static function; not some object method.
@@ -228,15 +225,14 @@ sub loop_do_timeslice {
   # there are any.  If nothing is waiting, set the timeout for some
   # constant number of seconds.
 
-  my $timeout = $_next_event_time;
-
   my $now = time();
+
+  my $timeout = $self->get_next_event_time();
   if (defined $timeout) {
     $timeout -= $now;
     $timeout = 0 if $timeout < 0;
   }
   else {
-    die "shouldn't happen" if ASSERT_DATA;
     $timeout = 3600;
   }
 
@@ -250,6 +246,8 @@ sub loop_do_timeslice {
     );
   }
 
+  my @filenos = %poll_fd_masks;
+
   if (TRACE_FILES) {
     foreach (sort { $a<=>$b} keys %poll_fd_masks) {
       my @types;
@@ -272,101 +270,99 @@ sub loop_do_timeslice {
     }
   }
 
-  # Avoid looking at filehandles if we don't need to.
-  # TODO The added code to make this sleep is non-optimal.  There is a
-  # way to do this in fewer tests.
+  # Avoid looking at filehandles if we don't need to.  TODO The added
+  # code to make this sleep is non-optimal.  There is a way to do this
+  # in fewer tests.
 
-  if (scalar keys %poll_fd_masks) {
+  if ($timeout or @filenos) {
 
     # There are filehandles to poll, so do so.
 
-    # Check filehandles, or wait for a period of time to elapse.
-    my $hits = IO::Poll::_poll($timeout * 1000, my @results = %poll_fd_masks);
+    if (@filenos) {
+      # Check filehandles, or wait for a period of time to elapse.
+      my $hits = IO::Poll::_poll($timeout * 1000, @filenos);
 
-    if (ASSERT_FILES) {
-      if ($hits < 0) {
-        POE::Kernel::_trap("<fh> poll returned $hits (error): $!")
-          unless ( ($! == EINPROGRESS) or
-                   ($! == EWOULDBLOCK) or
-                   ($! == EINTR) or
-                   ($! == 0)      # SIGNAL_PIPE strangeness
-                 );
+      if (ASSERT_FILES) {
+        if ($hits < 0) {
+          POE::Kernel::_trap("<fh> poll returned $hits (error): $!")
+            unless ( ($! == EINPROGRESS) or
+                     ($! == EWOULDBLOCK) or
+                     ($! == EINTR)
+                   );
+        }
       }
-    }
 
-    if (TRACE_FILES) {
+      if (TRACE_FILES) {
+        if ($hits > 0) {
+          POE::Kernel::_warn "<fh> poll hits = $hits\n";
+        }
+        elsif ($hits == 0) {
+          POE::Kernel::_warn "<fh> poll timed out...\n";
+        }
+      }
+
+      # If poll has seen filehandle activity, then gather up the
+      # active filehandles and synchronously dispatch events to the
+      # appropriate handlers.
+
       if ($hits > 0) {
-        POE::Kernel::_warn "<fh> poll hits = $hits\n";
-      }
-      elsif ($hits == 0) {
-        POE::Kernel::_warn "<fh> poll timed out...\n";
+
+        # This is where they're gathered.
+
+        my (@rd_ready, @wr_ready, @ex_ready);
+        while (@filenos) {
+          my ($fd, $got_mask) = splice(@filenos, 0, 2);
+          next unless $got_mask;
+
+          my $watch_mask = $poll_fd_masks{$fd};
+          if ( $watch_mask & POLLRDNORM and
+               $got_mask & (POLLRDNORM | POLLHUP | POLLERR | POLLNVAL)
+             ) {
+            if (TRACE_FILES) {
+              POE::Kernel::_warn "<fh> enqueuing read for fileno $fd";
+            }
+
+            push @rd_ready, $fd;
+          }
+
+          if ( $watch_mask & POLLWRNORM and
+               $got_mask & (POLLWRNORM | POLLHUP | POLLERR | POLLNVAL)
+             ) {
+            if (TRACE_FILES) {
+              POE::Kernel::_warn "<fh> enqueuing write for fileno $fd";
+            }
+
+            push @wr_ready, $fd;
+          }
+
+          if ( $watch_mask & POLLRDBAND and
+               $got_mask & (POLLRDBAND | POLLHUP | POLLERR | POLLNVAL)
+             ) {
+            if (TRACE_FILES) {
+              POE::Kernel::_warn "<fh> enqueuing expedite for fileno $fd";
+            }
+
+            push @ex_ready, $fd;
+          }
+        }
+
+        @rd_ready and $self->_data_handle_enqueue_ready(MODE_RD, @rd_ready);
+        @wr_ready and $self->_data_handle_enqueue_ready(MODE_WR, @wr_ready);
+        @ex_ready and $self->_data_handle_enqueue_ready(MODE_EX, @ex_ready);
       }
     }
 
-    # If poll has seen filehandle activity, then gather up the
-    # active filehandles and synchronously dispatch events to the
-    # appropriate handlers.
-
-    if ($hits > 0) {
-
-      # This is where they're gathered.
-
-      my (@rd_ready, @wr_ready, @ex_ready);
-      my %poll_fd_results = @results;
-      while (my ($fd, $got_mask) = each %poll_fd_results) {
-        next unless $got_mask;
-
-        my $watch_mask = $poll_fd_masks{$fd};
-        if (
-          $watch_mask & POLLRDNORM and
-          $got_mask & (POLLRDNORM | POLLHUP | POLLERR | POLLNVAL)
-        ) {
-          if (TRACE_FILES) {
-            POE::Kernel::_warn "<fh> enqueuing read for fileno $fd";
-          }
-
-          push @rd_ready, $fd;
-        }
-
-        if (
-          $watch_mask & POLLWRNORM and
-          $got_mask & (POLLWRNORM | POLLHUP | POLLERR | POLLNVAL)
-        ) {
-          if (TRACE_FILES) {
-            POE::Kernel::_warn "<fh> enqueuing write for fileno $fd";
-          }
-
-          push @wr_ready, $fd;
-        }
-
-        if (
-          $watch_mask & POLLRDBAND and
-          $got_mask & (POLLRDBAND | POLLHUP | POLLERR | POLLNVAL)
-        ) {
-          if (TRACE_FILES) {
-            POE::Kernel::_warn "<fh> enqueuing expedite for fileno $fd";
-          }
-
-          push @ex_ready, $fd;
-        }
-      }
-
-      @rd_ready and $self->_data_handle_enqueue_ready(MODE_RD, @rd_ready);
-      @wr_ready and $self->_data_handle_enqueue_ready(MODE_WR, @wr_ready);
-      @ex_ready and $self->_data_handle_enqueue_ready(MODE_EX, @ex_ready);
-    }
-  }
-  elsif ($timeout) {
-
     # No filehandles to poll on.  Try to sleep instead.  Use sleep()
     # itself on MSWin32.  Use a dummy four-argument select() everywhere
     # else.
 
-    if ($^O eq 'MSWin32') {
-      sleep($timeout);
-    }
     else {
-      CORE::select(undef, undef, undef, $timeout);
+      if ($^O eq 'MSWin32') {
+        sleep($timeout);
+      }
+      else {
+        CORE::select(undef, undef, undef, $timeout);
+      }
     }
   }
 
@@ -397,7 +393,7 @@ __END__
 
 =head1 NAME
 
-POE::Loop::IO_Poll - a bridge that allows POE to be driven by IO::Poll
+POE::Loop::IO_Poll - a bridge that supports IO::Poll from POE
 
 =head1 SYNOPSIS
 
@@ -405,13 +401,13 @@ See L<POE::Loop>.
 
 =head1 DESCRIPTION
 
-POE::Loop::IO_Poll implements the interface documented in L<POE::Loop>.
-Therefore it has no documentation of its own.  Please see L<POE::Loop>
-for more details.
+This class is an implementation of the abstract POE::Loop interface.
+It follows POE::Loop's public interface exactly.  Therefore, please
+see L<POE::Loop> for its documentation.
 
 =head1 SEE ALSO
 
-L<POE>, L<POE::Loop>, L<IO::Poll>, L<POE::Loop::PerlSignals>
+L<POE>, L<POE::Loop>, L<IO::Poll>
 
 =head1 AUTHORS & LICENSING
 
@@ -421,4 +417,4 @@ and POE's licensing.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
-# TODO - Edit.
+# TODO - Redocument.
