@@ -1,4 +1,4 @@
-# $Id: TkActiveState.pm 1980 2006-06-11 19:23:12Z rcaputo $
+# $Id: TkActiveState.pm 2222 2007-08-19 05:02:19Z rcaputo $
 
 # Tk-Perl event loop bridge for POE::Kernel.
 
@@ -6,7 +6,7 @@
 package POE::Loop::TkActiveState;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1980 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2222 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 # Merge things into POE::Loop::Tk.
 package POE::Loop::Tk;
@@ -117,7 +117,7 @@ sub loop_resume_filehandle {
 # This is the select loop itself.  We do a Bad Thing here by polling
 # for socket activity, but it's necessary with ActiveState's Tk.
 #
-# -><- We should really stop the poller when there are no handles to
+# TODO We should really stop the poller when there are no handles to
 # watch and resume it as needed.
 
 sub _poll_for_io {
@@ -142,7 +142,7 @@ sub _poll_for_io {
     );
   }
 
-  # Avoid looking at filehandles if we don't need to.  -><- The added
+  # Avoid looking at filehandles if we don't need to.  TODO The added
   # code to make this sleep is non-optimal.  There is a way to do this
   # in fewer tests.
 
@@ -152,7 +152,7 @@ sub _poll_for_io {
 
     if (@filenos) {
       # Check filehandles, or wait for a period of time to elapse.
-      my $hits = select(
+      my $hits = CORE::select(
         my $rout = $loop_vectors[MODE_RD],
         my $wout = $loop_vectors[MODE_WR],
         my $eout = $loop_vectors[MODE_EX],
@@ -281,3 +281,6 @@ Please see L<POE> for more information about authors, contributors,
 and POE's licensing.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
+# TODO - Redocument.
