@@ -1,9 +1,11 @@
+# $Id: Session.pm 2262 2007-12-12 03:18:11Z gwyn17 $
+
 package POE::Session;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.294_512'; # NOTE - Should be #.### (three decimal places)
+$VERSION = do {my($r)=(q$Revision: 2262 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Carp qw(carp croak);
 use Errno;
@@ -39,7 +41,7 @@ sub _define_assert {
   no strict 'refs';
   foreach my $name (@_) {
 
-    local $^W = 0;
+    BEGIN { $^W = 0 };
 
     next if defined *{"ASSERT_$name"}{CODE};
     if (defined *{"POE::Kernel::ASSERT_$name"}{CODE}) {
@@ -61,7 +63,7 @@ sub _define_assert {
 sub _define_trace {
   no strict 'refs';
 
-  local $^W = 0;
+  BEGIN { $^W = 0 };
 
   foreach my $name (@_) {
     next if defined *{"TRACE_$name"}{CODE};
@@ -669,7 +671,6 @@ sub get_heap {
 # steering me right on this one.
 
 my %anonevent_parent_id;
-my %anonevent_weakened;
 
 # I assume that when the postback owner loses all reference to it,
 # they are done posting things back to us.  That's when the postback's
@@ -678,19 +679,7 @@ my %anonevent_weakened;
 sub POE::Session::AnonEvent::DESTROY {
   my $self = shift;
   my $parent_id = delete $anonevent_parent_id{$self};
-  unless (delete $anonevent_weakened{$self}) {
-    $POE::Kernel::poe_kernel->refcount_decrement( $parent_id, 'anon_event' );
-  }
-}
-
-sub POE::Session::AnonEvent::weaken {
-  my $self = shift;
-  unless ($anonevent_weakened{$self}) {
-    my $parent_id = $anonevent_parent_id{$self};
-    $POE::Kernel::poe_kernel->refcount_decrement( $parent_id, 'anon_event' );
-    $anonevent_weakened{$self} = 1;
-  }
-  return $self;
+  $POE::Kernel::poe_kernel->refcount_decrement( $parent_id, 'anon_event' );
 }
 
 # Tune postbacks depending on variations in toolkit behavior.
@@ -753,6 +742,7 @@ sub callback {
   return $callback;
 }
 
+###############################################################################
 1;
 
 __END__
@@ -813,7 +803,7 @@ dispatch interface to those handlers.
 Consider the L</SYNOPSIS> for example.  A POE::Session instance is
 created with two C<inline_states>, each mapping an event name
 ("_start" and "next") to an inline subroutine.  POE::Session ensures
-that L</$_[KERNEL]> and so on are meaningful within an event handler.
+that C<$_[KERNEL]> and so on are meaningful within an event handler.
 
 Event handlers may also be object or class methods, using
 L</object_states> and L</package_states> respectively.  The create()
@@ -824,12 +814,12 @@ Notice that the created POE::Session object has not been saved to a
 variable.  The new POE::Session object gives itself to POE::Kernel,
 which then manages it and all the resources it uses.
 
-It's possible to keep references to new POE::Session objects, but it's
-not usually necessary.  If an application is not careful about
-cleaning up these references you will create circular references,
-which will leak memory when POE::Kernel would normally destroy the
-POE::Session object.  It is recommended that you keep the session's
-L<ID> instead.
+It's possible to keep references to new POE::Session objects, but it's 
+not usually necessary.  If an application is not careful about 
+cleaning up these references you will create circular references, 
+which will leak memory when POE::Kernel would normally destroy 
+the POE::Session object.  It is recommended that you keep 
+the session's C</ID> instead.
 
 =head2 POE::Session's Calling Convention
 
@@ -841,17 +831,16 @@ POE::Session's unconventional calling convention.  For example:
     ...;
   }
 
-Or the use of C<$_[KERNEL]>, C<$_[HEAP]> and C<$_[ARG0]> inline,
-as is done in most examples.
+Or the use of CL</$_[KERNEL]>, CL</$_[HEAP]> and CL</$_[ARG0]> inline, as is done
+in most examples.
 
 What's going on here is rather basic.  Perl passes parameters into
-subroutines or methods using the @_ array.  C<KERNEL>, C<HEAP>,
-C<ARG0> and others are constants exported by POE::Session (which is
-included for free when a program uses POE).
+subroutines or methods using the @_ array.  C<KERNEL>, C<HEAP>, C<ARG0> and
+others are constants exported by POE::Session (which is included for
+free when a program uses POE).
 
-So C<$_[KERNEL]> is an event handler's KERNELth parameter.
-C<@_[HEAP, ARG0]> is a slice of @_ containing the HEAPth and ARG0th
-parameters.
+So CL</$_[KERNEL]> is an event handler's KERNELth parameter.  C<@_[HEAP,
+ARG0]> is a slice of @_ containing the HEAPth and ARG0th parameters.
 
 While this looks odd, it's perfectly plain and legal Perl syntax.  POE
 uses it for a few reasons:
@@ -865,8 +854,8 @@ hash or array references and then dereferencing them in the handler.
 
 =item 2
 
-Typos in hash-based parameter lists are either subtle run-time
-errors or requires constant run-time checking.  Constants are either
+Typos in hash-based parameter lists are either subtle runtime
+errors or requires constant runtime checking.  Constants are either
 known at compile time, or are clear compile-time errors.
 
 =item 3
@@ -883,7 +872,7 @@ use only the parameters they're interested in.
 
 =head2 POE::Session Parameters
 
-Event handlers receive most of their run-time context in up to nine
+Event handlers receive most of their runtime context in up to nine
 callback parameters.  POE::Kernel provides many of them.
 
 =head3 $_[OBJECT]
@@ -906,14 +895,11 @@ methods in the same object.  C<$_[OBJECT]> helps this happen.
     $self->update_status_line();
   }
 
-You may also use method inheritance.  Here we invoke
-$self->a_method(@_).  Since Perl's C<<->>> operator unshifts $self
-onto the beginning of @_, we must first shift a copy off to maintain
-POE's parameter offsets:
+You may also use method inheritance
 
-  sub a_method {
+  sub Amethod {
     my $self = shift;
-    $self->SUPER::a_method( @_ );
+    $self->SUPER::Amethod( @_ );
     # ... more work ...
   }
 
@@ -921,11 +907,11 @@ POE's parameter offsets:
 
 C<$_[SESSION]> is a reference to the current session object.  This lets event
 handlers access their session's methods.  Programs may also compare
-C<$_[SESSION]> to C<$_[SENDER]> to verify that intra-session events did not
+C<$_[SESSION]> to CL</$_[SENDER]> to verify that intra-session events did not
 come from other sessions.
 
 C<$_[SESSION]> may also be used as the destination for intra-session
-L<post()|POE::Kernel/post> and L<call()|POE::Kernel/call>.  L<yield()|POE::Kernel/yield> is marginally more convenient and
+CL<post()|POE::Kernel/post> and CL<call()|POE::Kernel/call>.  CL<yield()|POE::Kernel/yield> is marginally more convenient and
 efficient than C<post($_[SESSION], ...)> however.
 
 It is bad form to access another session directly.  The recommended
@@ -1076,13 +1062,13 @@ what the heck---go ahead and mix in some inline code as well.
 However only one handler may be assigned to a given event name.
 Duplicates will overwrite earlier ones.
 
-event_1a is handled by calling C<< $object_1->method_1a(...) >>.  C<$_[OBJECT]>
+event_1a is handled by calling C<$object_1->method_1a(...)>.  C<$_[OBJECT]>
 is C<$object_1> in this case.  C<$_[HEAP]> belongs to the session, which
 means anything stored there will be available to any other event
 handler regardless of the object.
 
-event_2a is handled by calling C<< $object_2->method_2a(...) >>.  In this
-case C<$_[OBJECT]> is $object_2.  C<$_[HEAP]> is the same anonymous hashref
+event_2a is handled by calling C<$object_2->method_2a(...)>.  In this
+case CL</$_[OBJECT]> is $object_2.  C<$_[HEAP]> is the same anonymous hashref
 that was passed to the event_1a handler, though.  The methods are resolved
 when the event is handled (late-binding).
 
@@ -1101,8 +1087,8 @@ single object can't handle events from more than one session:
     );
   }
 
-Now C<< $object_4->method_4(...) >> may be called to handle events from one of
-two sessions.  In both cases, C<$_[OBJECT]> will be C<$object_4>, but
+Now C<$object_4->method_4(...)> may be called to handle events from one
+of two sessions.  In both cases, C<$_[OBJECT]> will be C<$object_4>, but
 C<$_[HEAP]> will hold data for a particular session.
 
 The same goes for inline states.  One subroutine may handle events
@@ -1122,7 +1108,7 @@ object upon success, but most applications won't need to save it.
 C<create()> invokes the newly started session's _start event handler
 before returning.
 
-C<create()> also passes the new POE::Session object to L<POE::Kernel>.
+C<create()> also passes the new POE::Session object to L</POE::Kernel>.
 POE's kernel holds onto the object in order to dispatch events to it.
 POE::Kernel will release the object when it detects the object has
 become moribund.  This should cause Perl to destroy the object if
@@ -1217,11 +1203,11 @@ Be very careful with closures, however.  L</Beware circular references>.
 
 C<object_states> associates one or more objects to a session and maps
 event names to the object methods that will handle them.  It's value
-is an C<ARRAYREF>; C<HASHREFs> would stringify the objects, ruining them
+is an CB<ARRAYREF>; C<HASHREFs> would stringify the objects, ruining them
 for method invocation.
 
-Here _start is handled by C<< $object->_session_start() >> and _stop triggers
-C<< $object->_session_stop() >>:
+Here _start is handled by C<$object->_session_start()> and _stop triggers
+C<$object->_session_stop()>:
 
   POE::Session->create(
     object_states => [
@@ -1243,7 +1229,7 @@ triggers $object->_stop():
   );
 
 Methods are verified when the session is created, but also resolved when the
-handler is called (late binding).  Most of the time, a method won't change.
+handler is called (late binding).  Most of the time, a method won't change. 
 But in some circumstance, such as dynamic inheritance, a method could
 resolve to a different subroutine.
 
@@ -1251,7 +1237,7 @@ resolve to a different subroutine.
 
 POE::Session sessions support a small number of options, which may be
 initially set with the C<option> constructor parameter and changed at
-run time with the C<option()|/option> mehtod.
+runtime with the C<option()|/option> mehtod.
 
 C<option> takes a hashref with option =E<gt> value pairs:
 
@@ -1283,11 +1269,11 @@ The value for C<package_states> is an B<ARRAYREF> to be consistent
 with C<object_states>, even though class names (also known as package names) are
 already strings, so it's not necessary to avoid stringifying them.
 
-Here _start is handled by C<< $class_name->_session_start() >> and _stop
-triggers C<< $class_name->_session_stop() >>:
+Here _start is handled by C<$class_name->_session_start()> and _stop
+triggers C<$class_name->_session_stop()>:
 
   POE::Session->create(
-    package_states => [
+    class_states => [
       $class_name => {
         _start => '_session_start',
         _stop  => '_session_stop',
@@ -1296,11 +1282,11 @@ triggers C<< $class_name->_session_stop() >>:
   );
 
 POE::Session also supports a short form where the event and method
-names are identical.  Here _start invokes C<< $class_name->_start() >>, and
-_stop triggers C<< $class_name->_stop() >>:
+names are identical.  Here _start invokes C<$class_name->_start()>, and
+_stop triggers C<$class_name->_stop()>:
 
   POE::Session->create(
-    package_states => [
+    class_states => [
       $class_name => [ '_start', '_stop' ],
     ]
   );
@@ -1319,8 +1305,8 @@ to store 4.29 billion session objects.
 
 A session's ID is unique within a running process, but multiple
 processes are likely to have the same session IDs.  If a global ID is
-required, it will need to include both C<< $_[KERNEL]->ID >> and
-C<< $_[SESSION]->ID >>.
+required, it will need to include both C<$_[KERNEL]->ID> and
+C<$_[SESSION]->ID>.
 
 =head2 option OPTION_NAME [, OPTION_VALUE [, OPTION_NAME, OPTION_VALUE]... ]
 
@@ -1434,17 +1420,9 @@ handle_ok_button():
   Postback created with (8 6 7).
   Postback called with (5 3 0 9).
 
+
 Postbacks hold references to their target sessions.  Therefore
-sessions with outstanding postbacks will remain active.  Under every
-event loop except Tk, postbacks are blessed so that DESTROY may be
-called when their users are done.  This triggers a decrement on their
-reference counts, allowing sessions to stop.
-
-Postbacks have one method, weaken(), which may be used to reduce their
-reference counts upon demand.  weaken() returns the postback, so you
-can do:
-
-  my $postback = $session->postback("foo")->weaken();
+sessions with outstanding postbacks will remain active.
 
 Postbacks were created as a thin adapter between callback libraries
 and POE.  The problem at hand was how to turn callbacks from the Tk
@@ -1469,8 +1447,8 @@ passed by the Gtk2 button will be in ARG1.
 
 =head2 callback EVENT_NAME, EVENT_PARAMETERS
 
-callback() manufactures callbacks that use C<< $poe_kernel->call() >> to
-deliver POE events rather than C<< $poe_kernel->post() >>.  It is identical
+callback() manufactures callbacks that use C<$poe_kernel->call()> to
+deliver POE events rather than C<$poe_kernel->post()>.  It is identical
 to C<postback()> in every other respect.
 
 callback() was created to avoid race conditions that arise when
@@ -1500,7 +1478,7 @@ Rather, helper functions may use:
   use POE::Kernel; # exports $poe_kernel
   sub some_helper_function {
     my (@specific_parameters) = @_;
-    my $session = $poe_kernel->get_active_session();
+    my $session = $kernel->get_active_session();
     my $heap = $session->get_heap();
   }
 
@@ -1523,7 +1501,7 @@ parameters to C<create()>.
 
 Any parameters not recognized by C<create()> must be removed from the
 CREATE_PARAMETERS before C<instantiate()> returns.  C<create()> will
-L<croak|Carp>
+L<croak|Carp/croak>
 if it discovers unknown parameters.
 
 Be sure to return C<$self> from instantiate.
@@ -1563,7 +1541,7 @@ that are passed to the session's _start handler.
     return $self->SUPER::try_alloc(@args);
   }
 
-=head1 POE::Session's EVENTS
+=head1 POE::Session'S EVENTS
 
 Please do not define new events that begin with a leading underscore.
 POE claims /^_/ events as its own.
@@ -1646,18 +1624,9 @@ value wraps.  This can occur after as I<few> as 4.29 billion sessions.
 
 =head2 Beware circular references
 
-As you're probably aware, a circular reference is when a variable is
-part of a reference chain that eventually refers back to itself.  Perl
-will not reclaim the memory involved in such a reference chain until
-the chain is manually broken.
+The following creates a circular reference:
 
-Here a POE::Session is created that refers to itself via an external
-scalar.  The event handlers import $session via closures which are in
-turn stored within $session.  Even if this session stops, the circular
-references will remain.
-
-  my $session;
-  $session = POE::Session->create(
+  my $session = POE::Session->create(
     inline_states => {
       _start => sub {
         $_[HEAP]->{todo} = [ qw( step1 step2 step2a ) ],
@@ -1672,81 +1641,33 @@ references will remain.
     }
   );
 
-Reduced to its essence:
-
-  my %event_handlers;
-  $event_handler{_start} = sub { \%event_handlers };
-
-Note also that a anonymous sub creates a closure on all lexical
-variables in the scope it was defined in, even if it doesn't reference
-them.  $session is still being held in a circular reference here:
+Note also that a anonymous sub creates a closure on all lexical variables in 
+the scope it was defined in, even if it doesn't reference them:
 
   my $self = $package->new;
-  my $session;
-  $session = POE::Session->create(
+  my $session = POE::Session->create(
     inline_state => {
       _start => sub { $self->_start( @_[ARG0..$#_] ) }
     }
   );
 
-To avoid this, a session may set an alias for itself.  Other parts of
-the program may then refer to it by alias.  In this case, one needn't
-keep track of the session themselves (POE::Kernel will do it anyway).
+To avoid this, must hold onto a session's ID, rather then the session's
+object.  You may convert a session ID back into the session's object with
+L<POE::Kernel->session_id_to_session()|POE::Kernel/session_id_to_session>.
 
-  POE::Session->create(
+  my $session = POE::Session->create(
     inline_states => {
       _start => sub {
+        $session = $session->ID;
         $_[HEAP]->{todo} = [ qw( step1 step2 step2a ) ],
-        $_[KERNEL]->alias_set('step_doer');
-        $_[KERNEL]->post( 'step_doer', 'next' );
+        $_[KERNEL]->post( $session, 'next' );
       },
-      next => sub {
-        my $next = shift @{ $_[HEAP]->{todo} };
-        return unless $next;
-        $_[KERNEL]->post( 'step_doer', $next );
-      }
       # ....
     }
   );
 
-Aliases aren't even needed in the previous example because the session
-refers to itself.  One could instead use POE::Kernel's yield() method
-to post the event back to the current session:
 
-  next => sub {
-    my $next = shift @{ $_[HEAP]->{todo} };
-    return unless $next;
-    $_[KERNEL]->yield( $next );
-  }
 
-Or the L</$_[SESSION]> parameter passed to every event handler, but
-yield() is more efficient.
-
-  next => sub {
-    my $next = shift @{ $_[HEAP]->{todo} };
-    return unless $next;
-    $_[KERNEL]->post( $_[SESSION], $next );
-  }
-
-Along the same lines as L</$_[SESSION]>, a session can respond back to
-the sender of an event by posting to L</$_[SENDER]>.  This is great for
-responding to requests.
-
-If a program must hold onto some kind of dynamic session reference,
-it's recommended to use the session's numeric ID rather than the
-object itself.  A session ID may be converted back into its object,
-but post() accepts session IDs as well as objects and aliases:
-
-  my $session_id;
-  $session_id = POE::Session->create(
-    inline_states => {
-      _start => sub {
-        $_[HEAP]->{todo} = [ qw( step1 step2 step2a ) ],
-        $_[KERNEL]->post( $session_id, 'next' );
-      },
-      # ....
-    }
-  )->ID;
 
 =head1 AUTHORS & COPYRIGHTS
 
@@ -1755,4 +1676,3 @@ Please see L<POE> for more information about authors and contributors.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
-# TODO - Edit.
