@@ -1,6 +1,5 @@
 #!/usr/bin/perl
-# $Id: Makefile-5005.pm 2391 2008-07-05 22:50:47Z rcaputo $
-# rocco // vim: ts=2 sw=2 expandtab
+# $Id: Makefile-5005.pm 2286 2008-03-10 08:32:44Z rcaputo $
 
 use strict;
 
@@ -23,11 +22,7 @@ use PoeBuildInfo qw(
 open(TOUCH, ">>CHANGES") and close TOUCH;
 open(TOUCH, ">>META.yml") and close TOUCH;
 
-### Touch gen-tests.perl so it always triggers.
-
-utime(time(), time(), "mylib/gen-tests.perl");
-
-### Some advisory dependency testing.
+### Some advisory dependancy testing.
 
 sub check_for_modules {
   my ($dep_type, @modules) = @_;
@@ -81,6 +76,10 @@ check_for_modules(
   "URI"             => 1.30,
 );
 
+### Generate dynamic test files.
+
+system($^X, "mylib/gen-tests.perl") and die "couldn't generate tests: $!";
+
 ### Generate Makefile.PL.
 
 sub MY::postamble {
@@ -131,12 +130,8 @@ WriteMakefile(
     TESTS => TEST_FILES,
   },
 
-  # Not executed on "make test".
-  PL_FILES       => {
-    'mylib/gen-tests.perl' => [ 'lib/POE.pm' ],
-  },
-
   # More for META.yml than anything.
+  PL_FILES       => { },
   NO_META        => 1,
   PREREQ_PM      => { CORE_REQUIREMENTS },
 );
