@@ -1,4 +1,4 @@
-# rocco // vim: ts=2 sw=2 expandtab
+# $Id: PoeBuildInfo.pm 2268 2008-01-12 12:36:44Z bingosnet $
 
 # Build information for POE.  Moved into a library so it can be
 # required by Makefile.PL and gen-meta.perl.
@@ -18,16 +18,8 @@ use vars qw(@EXPORT_OK);
   DIST_ABSTRACT
   DIST_AUTHOR
   RECOMMENDED_TIME_HIRES
-  CONFIG_REQUIREMENTS
-  REPOSITORY
-  HOMEPAGE
 );
 
-
-sub CONFIG_REQUIREMENTS () {
-  ("POE::Test::Loops"  => '1.036_314')
-}
-
 sub CORE_REQUIREMENTS () {
   (
     "Carp"              => 0,
@@ -44,15 +36,9 @@ sub CORE_REQUIREMENTS () {
       ? (
         "Win32::Console" => 0.031,
         "Win32API::File" => 0.05,
-        "Win32::Job"     => 0.03,
-        "Win32::Process" => 0,
-        "Win32"          => 0,
-      )
-      : (
-        "IO::Tty"        => 1.08, # avoids crashes on fbsd
       )
+      : ()
     ),
-    CONFIG_REQUIREMENTS,
   )
 }
 
@@ -65,31 +51,30 @@ sub DIST_AUTHOR () {
 }
 
 sub DIST_ABSTRACT () {
-  ( 'Portable, event-loop agnostic eventy networking and multitasking.' )
+  ( 'A portable networking and multitasking framework.' )
 }
 
 sub CLEAN_FILES () {
   my @clean_files = qw(
-    */*/*/*/*~
-    */*/*/*~
-    */*/*/*~
-    */*/*~
-    */*~
-    *~
-    META.yml
-    Makefile.old
     bingos-followtail
     coverage.report
     poe_report.xml
     run_network_tests
+    test-output.err
     t/20_resources/10_perl
     t/20_resources/10_perl/*
     t/20_resources/20_xs
     t/20_resources/20_xs/*
-    t/30_loops
-    t/30_loops/*
-    t/30_loops/*/*
-    test-output.err
+    t/30_loops/10_select
+    t/30_loops/10_select/*
+    t/30_loops/20_poll
+    t/30_loops/20_poll/*
+    t/30_loops/30_event
+    t/30_loops/30_event/*
+    t/30_loops/40_gtk
+    t/30_loops/40_gtk/*
+    t/30_loops/50_tk
+    t/30_loops/50_tk/*
   );
   "@clean_files";
 }
@@ -103,12 +88,4 @@ sub TEST_FILES () {
   "@test_files";
 }
 
-sub REPOSITORY () {
-  'https://github.com/rcaputo/poe'
-}
-
-sub HOMEPAGE () {
-  'http://poe.perl.org/'
-}
-
 1;
